(*

  SPINEL - knihovna formtu S65 ("A") / ASCII formt

  Vydavatel: Papouch s.r.o. - http://spinel.papouch.com/

  Popis komunikanho protokolu SPINEL, Formt 65

  Dotaz:
   PRE FRM ADR SIG INST DATA CR
  Odpov:
   PRE FRM ADR SIG ACK DATA CR

   PRE  - Prefix (znak *), viz obecn popis
   FRM  - slo formtu 65 (znak A), viz obecn popis
   ADR  - Adresa zazen, ktermu je posln dotaz nebo kter posl odpov.
          Pouv se ASCII kdovn (2 znaky) hexadecimlnho vyjden. Viz
          poznmka 1).
   SIG  - Podpis zprvy - libovoln znak. Tent znak, kter byl posln v dotazu, se
          vrt v odpovdi, m lze snadno rozpoznat, na kter dotaz odpov
          pila. (1 znak)
   INST - Kd instrukce  Instrukce modulu QUIDO jsou podrobn popsny dle.
          Pouv se ASCII kdovn (2 znaky) hexadecimlnho vyjden. Viz
          poznmka 1).
   ACK  - Potvrzen dotazu (Acknowledge) - Potvrzen dotazu, zda a jak byl proveden.
          ACK jsou z intervalu 00H a 0FH aby se odliily od kdu instrukce a maj
          nsledujc vznam:
           (00H) - ve v podku
           (02H) - neplatn kd instrukce
           (03H) - neplatn data
           (04H) - nepovolen zpis/pstup odmtnut
           (0EH) - automaticky vyslan instrukce - kontinuln men
                   Pouv se ASCII kdovn (2 znaky) hexadecimlnho vyjden.
                   viz poznmka 1).
   DATA - Data. Podrobn popsny dle pro kadou instrukci. Pouv se ASCII
          kdovn (2 znaky) hexadecimlnho vyjden. viz poznmka 1).
   CR   - Zakonovan znak (carriage return)

  Poznmky:
    1) Byte v rznch stech formtu 65 jsou peneny v ASCII kdovn
       hexadecimlnho vyjden (HESCI).
       Tedy napklad bajt 1A Hex (26 dekadicky) je penen jako
       ASCII 1 a ASCII A. Pi psan na klvesnici tedy napeme 1A.

    2) Adresa FFH je rezervovna pro broadcast. Pokud je v dotazu adresa FFH,
       zazen se chov tak, jako by byla uvedena jeho adresa. Na dotazy
       s touto adresou se nevrac dn odpov.
       [const ADR_Universal]

    3) Adresa FEH je univerzln adresa. Pokud je v dotazu adresa FEH, zazen
       se chov tak, jako by byla uvedena jeho adresa. V odpovdi zazen uvede
       skutenou prv nastavenou adresu. Univerzln adresa se pouv jen v
       ppadech, kdy je na lince pipojen jen jedno zazen.
       [const ADR_Broadcast]

*)

unit uSpinel65;

interface

uses uSpinelCore;

Const
  FRM               = #65;  // formt protokolu Spinel "A"

  s65_ADR_Universal = #$FE; // zazen kter obdr zprvu s touto adresou se chovaj
                            // tak, jako by byla uvedena jeho adresa. Ppadn odpov
                            // se odesl.

  s65_ADR_Broadcast = #$FF; // zazen kter obdr zprvu s touto adresou se chovaj
                            // tak, jako by byla uvedena jeho adresa. Ppadn odpov
                            // se neodesl.

type
  TSpinelFormat65=record
   PRE: Char;          // prefix = vdy "*"
   FRM: Byte;          // formt
   ADR: Char;          // adresa
   SIG: Byte;          // podpis
   INST_ACK: Byte;     // instrukce / potvrzen
   DATA: String;       // data
   CR: Char;           // sufix = vdy #13 (enter)
  end;

function sEncode_S65(ADR:Char; SIG:byte; INST:byte; DATA:String):String; overload;
function sEncode_S65(SIG:byte; INST:byte; DATA:String):String; overload;
function sEncode_S65(INST:byte; DATA:String):String; overload;
function sEncode_S65Broadcast(SIG:byte; INST:byte; DATA:String):String;

Function sDecode_S65(PACKET:String):TSpinelFormat65;
Function sGetPacket_S65(Var Data:String; DelData:Boolean=False):String;
Function sChecek_S65(PACKET:String):Word;

implementation

uses SysUtils, StrUtils;

function sEncode_S65(ADR:Char; SIG:byte; INST:byte; DATA:String):String;
var i:integer;
begin
  result:=PRE+FRM+IntToHex(Ord(ADR),2)+Char(SIG)+IntToHex(INST,2);
  For i:=1 to Length(DATA) do Result:=Result+IntToHex(Ord(DATA[i]),2);
  Result:=Result+CR;
end;

function sEncode_S65(SIG:byte; INST:byte; DATA:String):String; overload;
begin
  result:=sEncode_S65(s65_ADR_Universal,SIG,INST,DATA);
end;

function sEncode_S65(INST:byte; DATA:String):String; overload;
begin
  result:=sEncode_S65(s65_ADR_Universal,$FF,INST,DATA);
end;

function sEncode_S65Broadcast(SIG:byte; INST:byte; DATA:String):String;
begin
  result:=sEncode_S65(s65_ADR_Broadcast,SIG,INST,DATA);
end;

function sDecode_S65(PACKET:String):TSpinelFormat65;
begin
  try
    If Length(PACKET)<8 then raise Exception.Create('');
    Result.PRE:=PACKET[1];
    Result.FRM:=Ord(PACKET[2]);
    Result.ADR:=Char(StrToInt('$'+Copy(PACKET,3,2)));
    Result.SIG:=Ord(PACKET[5]);
    Result.INST_ACK:=StrToInt('$'+Copy(PACKET,6,2));
    Result.DATA:=Copy(PACKET,8,Length(PACKET)-8);
    Result.CR:=PACKET[Length(PACKET)];
  except
    Raise Exception.Create('Incorrect PACKET (Spinel format 65)!');
  end;
end;

Function sGetPacket_S65(Var Data:String; DelData:Boolean=False):String;
(*
 Pokud je na potku etezce "Data" platn paket ve formatu spinel S65,
 tj. odpovd dlka, prefix a sufix, bude vsledkem funkce pekoprovan
 st etzce "Data! s tmto paketem, jinak vrac przdn etzec.
   Delete - (true) = Odmae se ze vstupnho bufferu "Data" nalezen paket.
*)
begin
  Result:='';
  try
    If Length(Data)<8 then Exit;
    If Data[1]+Data[2]<>PRE+FRM then Exit;
    If PosEx(CR,Data,7)=0 then Exit;
    Result:=Copy(Data,1,PosEx(CR,Data,7));

    if DelData then Delete(Data,1,Length(Result));
  except
  end;
end;

Function sChecek_S65(PACKET:String):Word;
(*
 Chybov kdy funkce sCheck65 - vychazi z sCheck97
  0 - Ok
  1 - Pli krtk paket
  2 - Neplatn prefix paketu (PRE)
  3 - Paket m jin pznak formatu (FRM)
  4 - (pro Spinel format 97)
  5 - (pro Spinel format 97)
  6 - Neplatn koncov znak (CR)
*)
Begin
  Result:=0;
  If Length(PACKET)<8 then Result:=1 else
  If PACKET[1]<>PRE then Result:=2 else
  If PACKET[2]<>FRM then Result:=3 else
  If PACKET[Length(PACKET)]<>CR then Result:=6;
end;

end.
